<?php
class IRCClient
{
    var $irc_nick;
    var $irc_user;
    var $irc_name;
	var $irc_host;
    var $irc_server;
    var $irc_port;
    var $irc_password;

    var $socket;
    var $error;
	var $errno;	
	var $callback_handlers;
	var $fp;

    var $actual_nick;
    var $connected;
    var $return;

    function IRCClient ($nick, $user, $name, $host)
    {   
        $this->actual_nick = $nick;
        $this->irc_nick = $nick;
        $this->irc_user = $user;
        $this->irc_name = $name;
        $this->irc_host = $host;
        $this->connected = FALSE;
        $this->socket   = NULL;
        $this->error    = NULL;
        $this->errno    = NULL;	
        $this->return   = NULL;
        $this->callback_handlers = array();
    }

    function irc_connect ($server, $pass=NULL)
    {
        $this->fp = fopen ('/tmp/irclog.log', 'a');

        list ($server, $port) = explode(':', $server);
        $this->irc_server = $server;
        $this->irc_port = ($port)? $port: '6667';
        $this->irc_pass = ($pass)? $pass: NULL;

        fwrite($this->fp, sprintf( "[%s] Connecting to %s (%s) port %s\n",
		                          date("YmdHis"), $this->irc_server,
					  gethostbyname($this->irc_server), $this->irc_port ) );
								  
        $this->socket = fsockopen ($this->irc_server, $this->irc_port,
            $this->errno, $this->error, 10);

        if(!$this->socket)
        {
           //echo "DEBUG: Socket is false<BR>";
           return(FALSE);
        }

        $this->connected = TRUE;

        set_socket_blocking($this->socket, FALSE);
        if ( !empty ($this->irc_password) ) {
            $this->irc_send_msg (sprintf ("PASS %s", $this->irc_password));
        }

        $this->irc_send_msg (sprintf ("USER %s %s %s :%s", $this->irc_user, $this->irc_user, $this->irc_user, $this->irc_name));
        $this->irc_send_msg (sprintf ("NICK %s", $this->irc_nick));

		
	    $this->irc_do_loop();
        return(TRUE);
    }

    function disconnect ($return = NULL)
    {
        $this->irc_send_msg("QUIT :$return");
		fclose ($this->socket);
		$this->socket = NULL;
		
		fwrite($this->fp, sprintf( "[%s] Disconnected from %s (%s) port %s\n",
		                          date("YmdHis"), $this->irc_server,
								  gethostbyname($this->irc_server), $this->irc_port ) );
		fclose ($this->fp);
        //echo "DEBUG: Processing a disconnect, setting connected=FALSE<BR>";
        $this->connected = FALSE;
        if($return != NULL)
           $this->return = $return;
    }

    function irc_parse_line($bufr)
    {
        $data = explode (" ", rtrim ($bufr, " \n"), 4);
        for ($i = 0; $i < count ($data); $i++)
        {
            $data[$i] = ltrim ($data[$i], ':');
        }
        if ( $data[0] == 'PING' ) 
        {
            $this->irc_send_msg (sprintf ("PONG %s", $data[1]), FALSE);
        }
        else if ( array_key_exists ($data[1], $this->callback_handlers) )
        {
            $this->callback_handlers[$data[1]] ( new irc_message($bufr), &$this );
        }
    }

    function irc_do_loop ()
    {
        $begin_time = time();
        $bigbuf = '';
        while ($this->connected == TRUE)
        {
            /*if(feof($this->socket))
               echo "DEBUG: socket is EOF<BR>";
            */
            $bufr = fgets ($this->socket, 1024); /* fgets stops on a newline */
            /*
            if($bufr == EOF)
            {
               echo "DEBUG: fgets returned EOF<BR>";
               $this->connected = FALSE;
               break;
            }
            */
            /* If this big has a end of line feed in it, we want to process it.
             * if not, its just a partial, save it and try again */
            if(preg_match('/[\n]$/', $bufr))
            {
                $bufr = $bigbuf . $bufr;
                $bigbuf = '';
		        fwrite($this->fp, sprintf( "[%s] %s", date("YmdHis"), $bufr));
                // DEBUG: printf("[%s] <- %s<BR>\n", date("YmdHis"), $bufr);
                $this->irc_parse_line($bufr);

            }
            else
            {
                $bigbuf = $bufr;
		        fwrite($this->fp, sprintf( "[%s] %s\n", date("YmdHis"), "No data yet, sleeping.."));
                fflush($this->fp);
                sleep(2);
            }
            if(time() - $begin_time > 20)
            {
                set_time_limit(30); // dont time out php
                $begin_time = time();
                if(array_key_exists('timeout', $this->callback_handlers))
                {
                    $this->callback_handlers['timeout'] (null, &$this);
                }
            }
        }
    }
	
    /* 
    :Sync.US.AfterNET.Org PONG Sync.US.AfterNET.Org :LAG2782252449
    :Sync.US.AfterNET.Org 381 Rubin :You are now an IRC Operator
    :AuthServ!AuthServ@X3.AfterNET.Services NOTICE Rubin :Rubin is authenticated to account Rubin.
    */
    function irc_register_callback ($event, $callback)
    {
        $this->callback_handlers[$event] = $callback;
    }	
	
    function irc_unregister_callback ($event)
    {
        $this->callback_handlers[$event] = NULL;
    }	
	
    function irc_send_msg ($msg)
    {
        set_socket_blocking($this->socket, TRUE);
	    fwrite($this->fp, sprintf( "[%s] %s\n", date("YmdHis"), $msg));
        // DEBUG: printf( "[%s] -> %s<BR>\n", date("YmdHis"), $msg);
        fputs ($this->socket, sprintf ("%s\n", $msg));
        set_socket_blocking($this->socket, FALSE);

    }
}

class irc_message
{
    var $nick;
    var $user;
    var $host;
    var $source;
    var $target;
    var $command;
    var $message;
    var $raw;

    function irc_message($raw)
    {
        $this->raw = $raw;
        if($raw)
        {
            if($raw[0] == ':') /* Normal ':source something dest :message' message */
            {
                /* Cut off the leading : */
                $raw = ltrim($raw, ':');
                $parts = explode(':', $raw);
                /* $parts[0] = 'source something dest' */
                list($source, $command, $destination) = explode(' ', $parts[0]);
                if($source)
                {
                    $this->source = $source;
                    if(preg_match('/^([^!]+)\!([^@]+)\@(.+)$/', $source, $matches))
                    {
                       $this->nick = $matches[1];
                       $this->user = $matches[2];
                       $this->host = $matches[3];
                    }
                    else
                    {
                        $this->host = $source; /* A server (?) */
                    }
                }
                if($command)
                  $this->command = $command;
                if($destination)
                  $this->target = $destination;
                /* $parts[1] = 'message' */
                $this->message = $parts[1];
            }
            else /* Direct 'PING :03403023' message */
            {
               /* Leave in $raw and thats it */
            }
        }
    }

    function matches($source, $command, $target, $message)
    {
        if(glob_match($source, $this->source) and glob_match($command, $this->command) and glob_match($target, $this->target) and glob_match($message, $this->message))
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
    function raw_matches($mask)
    {
        return glob_match($mask, $this->raw);
    }
}

function glob_match($mask, $string)
{
    /* Convert mask into regexp via 2 replacements: */
    $mask = preg_quote($mask);
    $mask = str_replace(array("\*", "\?"), array(".*","."), $mask);
    if(preg_match("/^". $mask . "$/i", $string))
        return TRUE;
    else
        return FALSE;
}
?>
